-- This software and documentation is Copyright (c) 2008 Geoff Richards
-- <geoff@laxan.com>.  It is free software; you can redistribute it
-- and/or modify it under the terms of the S<Lua 5.0> license.  The full terms
-- are given in the file I<COPYRIGHT> supplied with the source code package,
-- and are also available here: http://www.lua.org/license.html

local M = { _NAME = "issn" }
M.__index = M

local function _calc_checksum (data)
    local sum = 0
    for i = 1, 7 do
        sum = sum + tonumber(data:sub(i, i)) * (9 - i)
    end
    sum = (11 - sum % 11) % 11
    return (sum == 10) and "X" or tostring(sum)
end

local function _parse_issn (self, correction_mode)
    local issn = self._issn
    local expected_checksum = _calc_checksum(issn)
    if issn:sub(-1) ~= expected_checksum then
        if correction_mode == "fix-checksum" then
            self._issn = issn:sub(1, -2) .. expected_checksum
        else
            return "wrong checksum digit"
        end
    end
end

local function _new (class, input_data, correction_mode)
    if not correction_mode then correction_mode = "normal" end

    input_data = input_data:upper()             -- we want uppercase X's
    if correction_mode == "strict" then
        input_data = input_data:gsub("[-%s]", "")
        if input_data:find("[^0-9X]") then
            return nil, "character not allowed"
        end
    else
        input_data = input_data:gsub("[^0-9X]", "")
    end

    local len = input_data:len()
    if correction_mode == "fix-checksum" and len == 7 then
        -- allow checksum to be missing if we're going to correct it anyway.
        input_data = input_data .. "0"
    elseif len ~= 8 then
        return nil, "wrong number of digits"
    end
    if not input_data:find("^%d+[%dX]$") then
        return nil, "digit 'X' not allowed in middle"
    end

    local o = setmetatable({ _issn = input_data }, class)

    local err = _parse_issn(o, correction_mode)
    if err then return nil, err end

    return o
end

function M:issn () return self._issn end

function M:checksum () return self._issn:sub(-1) end

function M:__tostring ()
    local issn = self._issn
    return issn:sub(1, 4) .. "-" .. issn:sub(5, 8)
end

function M:eq (other)
    if type(other) == "string" then
        local tmp = _new(M, other)
        if not tmp then error("invalid ISSN value '" .. other .. "'", 2) end
        other = tmp
    end

    local selfstr, otherstr = self._issn, other._issn
    return selfstr == otherstr
end
M.__eq = M.eq

return setmetatable(M, { _NAME = "issn metatable", __call = _new })
-- vi:ts=4 sw=4 expandtab
