require "isbn-test"
local ISSN = require "issn"

module("test.issn", lunit.testcase, package.seeall)

local good_issn_digits = {
    "02613077", -- The Guardian
    "14734966", -- Photography Monthly

    -- From the Wikipedia article on ISSN.
    "03178471",
    "15340481",

    -- From RFC 3044 section 5.
    "0259000X",
    "15601560",
}

function test_module_metadata ()
    assert_equal("issn", ISSN._NAME, "_NAME")
end

function test_stringify ()
    local obj = assert(ISSN("0259000x"))
    assert_equal("0259000X", obj:issn(), ":issn()")
    assert_equal("0259-000X", tostring(obj), "tostring()")
    assert_equal("X", obj:checksum(), ":checksum()")
end

function test_mode ()
    local obj = assert(ISSN("\t02 59-00*0X\n"))
    assert_table(obj, "no mode")
    assert_equal("0259-000X", tostring(obj))
    obj = assert(ISSN("\t02 59-00*0X\n", "normal"))
    assert_table(obj, "normal mode")
    assert_equal("0259-000X", tostring(obj))
    obj = assert(ISSN("\t02 59-00*0X\n", "fix-checksum"))
    assert_table(obj, "fix-checksum mode")
    assert_equal("0259-000X", tostring(obj))
    obj = assert(ISSN("\t02 59-000X\n", "strict"))
    assert_table(obj, "strict mode")
    assert_equal("0259-000X", tostring(obj))
end

function test_bad_syntax ()
    local ok, err = ISSN("\t02 59-00*0X\n", "strict")
    assert_nil(ok, "strict with bad char")
    assert_equal("character not allowed", err)
    ok, err = ISSN("0259000")
    assert_nil(ok, "wrong number of digits")
    assert_equal("wrong number of digits", err)
    ok, err = ISSN("0259X000")
    assert_nil(ok, "digit 'X' in middle")
    assert_equal("digit 'X' not allowed in middle", err)
end

function test_eq ()
    local obj1 = assert(ISSN("02613077"))
    local obj2 = assert(ISSN("02613077"))
    assert_true(obj1:eq(obj2), "two objects equal")
    assert_true(obj1:eq("02613077"), "object and string equal")
    assert_true(obj1 == obj2, "two objects equal, == operator")
    assert_false(obj1 ~= obj2, "two objects equal, ~= operator")

    obj2 = assert(ISSN("15601560"))
    assert_false(obj1:eq(obj2), "two objects not equal")
    assert_false(obj1:eq("15601560"), "object and string not equal")
    assert_false(obj1 == obj2, "two objects not equal, == operator")
    assert_true(obj1 ~= obj2, "two objects not equal, ~= operator")
end

function test_bad_checksum ()
    for _, issn in ipairs(good_issn_digits) do
        local digits, good_checksum = issn:sub(1, 7), issn:sub(8, 8)
        good_checksum = (good_checksum == "X") and 10 or tonumber(good_checksum)
        for i = 0, 10 do
            if i ~= good_checksum then
                local input = digits .. (i == 10 and "X" or i)

                -- Detect bad checksum.
                for mode in ipairs({ "normal", "strict" }) do
                    local ok, err = ISSN(input, mode)
                    local msg = "detect error in '" .. input ..
                                "', mode '" .. mode .. "'"
                    assert_nil(ok, msg)
                    assert_equal("wrong checksum digit", err, msg)
                end

                -- Correct checksum.
                local obj = assert(ISSN(input, "fix-checksum"))
                local msg = "corrected checksum in '" .. input .. "'"
                assert_table(obj, msg)
                assert_equal(issn:sub(8, 8), obj:checksum(), msg)
                assert_equal(issn, obj:issn(), msg)
            end
        end
    end
end

-- vi:ts=4 sw=4 expandtab
